
module JBB_IPO

	OSX = ( Object::RUBY_PLATFORM =~ /darwin/i ? true : false )
	WIN = ( (Object::RUBY_PLATFORM =~ /mswin/i || Object::RUBY_PLATFORM =~ /mingw/i) ? true : false )

	if OSX
		@@MouseTools = File.join(File.dirname(__FILE__), '.bin/MouseTools')
		File.chmod(0755, @@MouseTools) unless File.executable?(@@MouseTools)
	end

	if WIN
		if RUBY_VERSION.to_i < 2
			require 'jbb_layers_panel/z_win32api/Win32API.so'
		else
			require 'Win32API'
		end#if
	end#if

	#-------------------------------------------------------------------------------


	def self.getScreenCoords
		coords = []
		if OSX
			@all_coords = %x(defaults  read  com.sketchup.SketchUp.20#{Sketchup.version.to_i}  "NSWindow Frame SketchUpDoc").chomp!.split(' ').each{|n| n.to_i}
			(coords << @all_coords[4..-1]).flatten
		elsif WIN
			metrics = Win32API.new("user32", "GetSystemMetrics", 'I', 'I')
			#Coordinates of the whole virtualScreen
			coords << metrics.call(76)
			coords << metrics.call(77)
			coords << metrics.call(59)
			coords << metrics.call(60)
		end#if
		coords #[left, top, right, bottom]
	end#def

	def self.getWindowCoords
		screenCoords = self.getScreenCoords.flatten
		if OSX
			view  = Sketchup.active_model.active_view
			sh = view.vpheight # start_height
			# sw  = view.vpwidth.freeze # start_width
			# mn_bar = 22  # menu bar always there, not accounted for
			# tb_bot = 31  # bottom toolbar is always there
			# @all ["38", "613", "512", "565", "0", "0", "1920", "1178"]
			# screenCoords ["0", "0", "1920", "1178"]
			# coords       ["38", "613", "512", "565"] scrn lft to su lft, scrn bot to su bot, su wdth, su bot to su top
			coords = (@all_coords[0..3]).flatten

			lft = [coords[0].to_i, screenCoords[0].to_i].max

			rgt = [coords[2].to_i, screenCoords[2].to_i].min + coords[0].to_i

			bot = [(screenCoords[3].to_i - coords[1].to_i), screenCoords[3].to_i].min

			top = bot - sh #(screenCoords[3].to_i - coords[1].to_i - coords[3].to_i)

			coords = [lft.to_i, top.to_i, rgt.to_i, bot.to_i]
		elsif WIN
			hwnd = Win32API.new("user32.dll", "GetActiveWindow", '', 'L').call
			coords = "0"*16
			Win32API.new("user32.dll", "GetWindowRect", 'LP', 'B').call(hwnd, coords)
			coords = coords.unpack('l*') #[left, top, right, bottom]
			#Make sure coordinates are inside screen boundaries
			coords[0] = screenCoords[0] if coords[0] < screenCoords[0]
			coords[1] = screenCoords[1] if coords[1] < screenCoords[1]
			coords[2] = screenCoords[2] if coords[2] > screenCoords[2]
			coords[3] = screenCoords[3] if coords[3] > screenCoords[3]
		end#if
		coords #[left, top, right, bottom]
	end#def

	def self.getMouseLocation
		if OSX
			result = %x("#{@@MouseTools}" -location).chomp!.split(' ').each{|n| n.to_i}
			x, y = result[0].to_i,result[1].to_i
		elsif WIN
			result = "0"*8
			Win32API.new("user32","GetCursorPos", 'P', 'V').call(result)
			x, y = result.unpack("LL")
		end#if
	end#def

	def self.setMouseLocation(x, y)
		#Save camera position
		view = Sketchup.active_model.active_view
		camera = view.camera
		eye = camera.eye
		target = camera.target
		up = camera.up

		#Move cursor to the other side of the SU window
		if OSX
			%x("#{@@MouseTools}" -x #{x} -y #{y} ) # -mouseSteps 100)
		elsif WIN
			setCursorPos = Win32API.new("user32", "SetCursorPos", 'II', 'V')
			setCursorPos.Call(x, y)
		end#if

		#Revert back the camera position
		#Timer required, otherwise the position is not reverted
		done = false
		timer = UI.start_timer(0, false) {
			next if done
			done = true
			camera.set(eye, target, up)
		}
	end#def


	#-------------------------------------------------------------------------------


	class JBB_ViewObserver < Sketchup::ViewObserver
		def onViewChanged(view)
			if Sketchup.read_default("jbb_infinitePanOrbit", "active") != false
				if Sketchup.active_model.tools.active_tool_name == "CameraOrbitTool"  || Sketchup.active_model.tools.active_tool_name == "CameraDollyTool"
					mouseLocation = JBB_IPO.getMouseLocation
					windowCoords = JBB_IPO.getWindowCoords
					
					if mouseLocation.x > windowCoords[2] - 20 #Right
						JBB_IPO.setMouseLocation(windowCoords[0] + 30, mouseLocation.y)
					elsif mouseLocation.x < windowCoords[0] + 20 #Left
						JBB_IPO.setMouseLocation(windowCoords[2] - 30, mouseLocation.y)
					end#if

					if mouseLocation.y > windowCoords[3] - 20 #Bottom
						JBB_IPO.setMouseLocation(mouseLocation.x, windowCoords[1] + 30)
					elsif mouseLocation.y < windowCoords[1] + 20 #Top
						JBB_IPO.setMouseLocation(mouseLocation.x, windowCoords[3] - 30)
					end#if
					#+-20 and +-30 are safe margins
				end#if
			end#if
		end#def
	end#class
	Sketchup.active_model.active_view.add_observer(JBB_ViewObserver.new)

	class JBB_AppObserver < Sketchup::AppObserver
		def onNewModel(newModel)
			newModel.active_view.add_observer(JBB_IPO::JBB_ViewObserver.new)
		end#def
		def onOpenModel(newModel)
			newModel.active_view.add_observer(JBB_IPO::JBB_ViewObserver.new)
		end#def
	end#class
	Sketchup.add_observer(JBB_AppObserver.new)


	#-------------------------------------------------------------------------------


	def self.toggle_infinitePanOrbit
		if Sketchup.read_default("jbb_infinitePanOrbit", "active") != false
			Sketchup.write_default("jbb_infinitePanOrbit", "active", false)
		else
			Sketchup.write_default("jbb_infinitePanOrbit", "active", true)
		end#if
	end#def

	def self.validation_proc
		if Sketchup.read_default("jbb_infinitePanOrbit", "active") != false
			MF_CHECKED
		else
			MF_UNCHECKED
		end#if
	end#def

	unless file_loaded?( __FILE__ )
		# Commands
		cmd = UI::Command.new( 'InfinitePanOrbit' ) { self.toggle_infinitePanOrbit }
		cmd.status_bar_text = 'Toggle InfinitePanOrbit'
		cmd.set_validation_proc { self.validation_proc }
		cmd_toggle_infinitePanOrbit = cmd

		menu = UI.menu("View")
		menu.add_item( cmd_toggle_infinitePanOrbit )
	end


	#-------------------------------------------------------------------------------


	#Sketchucation API
	begin
		SCFapi.store_event('JiBiBojabxz41322768h', 'InfinitePanOrbit', 'Load')
		if Sketchup.read_default("jbb_infinitePanOrbit", "active") != false
			SCFapi.store_event('JiBiBojabxz41322768h', 'InfinitePanOrbit', 'activeLoad')
		elsif  Sketchup.read_default("jbb_infinitePanOrbit", "active") == false
			SCFapi.store_event('JiBiBojabxz41322768h', 'InfinitePanOrbit', 'inactiveLoad')
		end#if
		SCFapi.send_events('JiBiBojabxz41322768h')
	rescue
	end

end#module
